# Testfälle add(numbers: str) -> int – StringCalculator

# 1. Leerer String
# Eingabe: "" → Erwartete Ausgabe: 0

# 2. Einzelne Zahl
# Eingabe: "1" → Erwartete Ausgabe: 1

# 3. Zwei Zahlen, getrennt durch Komma
# Eingabe: "1,2" → Erwartete Ausgabe: 3

# 4. Mehr als zwei Zahlen (beliebig viele erlaubt)
# Eingabe: "1,2,3" → Erwartete Ausgabe: 6
# Eingabe: "1,2,3,4,5" → Erwartete Ausgabe: 15

# 5. Leere Einträge ignorieren
# Eingabe: "1,,2" → Erwartete Ausgabe: 3
# Eingabe: "1, ,2" → Erwartete Ausgabe: 3
# Eingabe: ",,1,,2,,," → Erwartete Ausgabe: 3

# 6. Zeilenumbrüche als Trennzeichen erlauben
# Eingabe: "1\n2,3" → Erwartete Ausgabe: 6
# Eingabe: "1,2\n3" → Erwartete Ausgabe: 6
# Eingabe: "1\n2\n3" → Erwartete Ausgabe: 6

# 8. Negative Zahlen → Exception mit allen negativen Werten
# Eingabe: "1,-2" → Exception mit Message "negatives not allowed: -2"
# Eingabe: "-1,-2,3" → Exception mit Message "negatives not allowed: -1, -2"

# 9. Benutzerdefinierter Delimiter über Präfix: "//<delimiter>\n"
# Eingabe: "//;\n1;2" → 3
# Eingabe: "//|\n4|5|6" → 15
# Eingabe: "//_\n7_8_9" → 24

# 10. Zahlen größer als 1000 ignorieren
# Eingabe: "2,1001" → 2
# Eingabe: "1000,1" → 1001 (1000 zählt noch mit)
# Eingabe: "1234,1001,3" → 3

# 11. Delimiter mit beliebiger Länge (Format: //[delimiter]\n)
# Eingabe: "//[***]\n1***2***3" → 6
# Eingabe: "//[abc]\n4abc5abc6" → 15
# Eingabe: "//[+]\n1+2+3" → 6

import unittest
from src.stringcalculator import StringCalculator

class TestStringCalculator(unittest.TestCase):
    def setUp(self):
        # Vor jedem Test wird eine Instanz des Calculators erstellt
        self.calc = StringCalculator()

    def test_add_empty_string(self):
        # Leerer String → 0
        self.assertEqual(self.calc.add(""), 0)

    def test_add_single_number(self):
        # Ein einzelner Wert → int
        self.assertEqual(self.calc.add("1"), 1)

    def test_add_two_numbers(self):
        # Zwei Zahlen → Summe
        self.assertEqual(self.calc.add("1,2"), 3)

    def test_add_multiple_numbers(self):
        # Mehr als zwei Zahlen → Summe
        self.assertEqual(self.calc.add("1,2,3"), 6)
        self.assertEqual(self.calc.add("1,2,3,4,5"), 15)

    def test_add_ignores_empty_entries(self):
        # Leere Einträge ignorieren → Summe
        self.assertEqual(self.calc.add("1,,2"), 3)
        self.assertEqual(self.calc.add("1, ,2"), 3)
        self.assertEqual(self.calc.add(",,1,,2,,,"), 3)

    def test_add_with_newlines_between_numbers(self):
        # Zeilenumbrüche als Trennzeichen erlauben → Summe
        self.assertEqual(self.calc.add("1\n2,3"), 6)
        self.assertEqual(self.calc.add("1,2\n3"), 6)
        self.assertEqual(self.calc.add("1\n2\n3"), 6)

    def test_add_raises_exception_on_negative_number(self):
        # Negative Zahlen → Exception mit allen negativen Werten
        with self.assertRaises(ValueError) as context:
            self.calc.add("1,-2")
        self.assertIn("negatives not allowed: -2", str(context.exception))

    def test_add_raises_exception_on_multiple_negatives(self):
        # Negative Zahlen → Exception mit allen negativen Werten
        with self.assertRaises(ValueError) as context:
            self.calc.add("-1,-2,3")
        self.assertIn("negatives not allowed: -1, -2", str(context.exception))

    def test_add_with_custom_delimiter(self):
        # Benutzerdefinierter Delimiter über Präfix: "//<delimiter>\n"
        self.assertEqual(self.calc.add("//;\n1;2"), 3)
        self.assertEqual(self.calc.add("//|\n4|5|6"), 15)
        self.assertEqual(self.calc.add("//_\n7_8_9"), 24)

    def test_add_ignores_numbers_greater_than_1000(self):
        # Zahlen größer als 1000 ignorieren
        self.assertEqual(self.calc.add("2,1001"), 2)
        self.assertEqual(self.calc.add("1000,1"), 1001)
        self.assertEqual(self.calc.add("1234,1001,3"), 3)

    def test_add_with_multi_char_delimiter(self):
        # Delimiter mit beliebiger Länge (Format: //[delimiter]\n)
        self.assertEqual(self.calc.add("//[***]\n1***2***3"), 6)
        self.assertEqual(self.calc.add("//[abc]\n4abc5abc6"), 15)
        self.assertEqual(self.calc.add("//[+]\n1+2+3"), 6)

if __name__ == "__main__":
    unittest.main()

# python -m unittest testsOthers.testsOrlando.OrlandoTestStringCalculator -v